@@ -13,6 +13,7 @@ import string
 import sys
 import tempfile
 import types
+import getpass
 
 try:
     # Python < 2.7 annd Python >= 3.0 < 3.1
@@ -21,7 +22,6 @@ except ImportError:
     import unittest
 
 import keyring.backend
-from keyring.backend import PasswordSetError
 
 ALPHABET = string.ascii_letters + string.digits
 DIFFICULT_CHARS = string.whitespace + string.punctuation
@@ -94,12 +94,10 @@ def random_string(k, source = ALPHABET):
 
 def is_win32_crypto_supported():
     try:
-        from keyring.backends import win32_crypto
-        if sys.platform in ['win32'] and sys.getwindowsversion()[-2] == 2:
-            return True
+        __import__('keyring.backends.win32_crypto')
     except ImportError:
-        pass
-    return False
+        return False
+    return sys.platform in ['win32'] and sys.getwindowsversion()[-2] == 2
 
 def is_osx_keychain_supported():
     return sys.platform in ('mac','darwin')
@@ -112,8 +110,9 @@ def is_kwallet_supported():
 
 def is_crypto_supported():
     try:
-        from Crypto.Cipher import AES
-        import crypt
+        __import__('Crypto.Cipher.AES')
+        __import__('Crypto.Protocol.KDF')
+        __import__('Crypto.Random')
     except ImportError:
         return False
     return True
@@ -126,20 +125,28 @@ def is_gnomekeyring_supported():
 
 def is_qt4_supported():
     try:
-        from PyQt4.QtGui import QApplication
+        __import__('PyQt4.QtGui')
     except ImportError:
         return False
     return True
 
 def is_winvault_supported():
     try:
-        from keyring.backend import WinVaultKeyring
-        if sys.platform in ['win32'] and sys.getwindowsversion().major >= 6:
-            return True
+        __import__('win32cred')
+        has_pywin32 = True
     except ImportError:
-        pass
-    return False
+        has_pywin32 = False
+    return (
+        sys.platform in ['win32'] and sys.getwindowsversion().major >= 6
+        and has_pywin32
+    )
 
+def is_dbus_supported():
+    try:
+        __import__('dbus')
+    except ImportError:
+        return False
+    return True
 
 class BackendBasicTests(object):
     """Test for the keyring's basic funtions. password_set and password_get
@@ -356,7 +363,6 @@ class FileKeyringTests(BackendBasicTests):
 
 class UncryptedFileKeyringTestCase(FileKeyringTests, unittest.TestCase):
 
-
     def init_keyring(self):
         return keyring.backend.UncryptedFileKeyring()
 
@@ -367,7 +373,13 @@ class CryptedFileKeyringTestCase(FileKeyringTests, unittest.TestCase):
 
     def setUp(self):
         super(self.__class__, self).setUp()
-        self.keyring._getpass = lambda *args, **kwargs: "abcdef"
+        # patch the getpass module to bypass user input
+        self.getpass_orig = getpass.getpass
+        getpass.getpass = lambda *args, **kwargs: "abcdef"
+
+    def tearDown(self):
+        getpass.getpass = self.getpass_orig
+        del self.getpass_orig
 
     def init_keyring(self):
         return keyring.backend.CryptedFileKeyring()
@@ -395,10 +407,30 @@ class WinVaultKeyringTestCase(BackendBasicTests, unittest.TestCase):
     def init_keyring(self):
         return keyring.backend.WinVaultKeyring()
 
+@unittest.skipUnless(is_dbus_supported(),
+    "DBus needed for SecretServiceKeyring")
+class SecretServiceKeyringTestCase(BackendBasicTests, unittest.TestCase):
+    __test__ = True
+
+    def environ(self):
+        return dict(DISPLAY='1',
+                    DBUS_SESSION_BUS_ADDRESS='1')
+
+    def init_keyring(self):
+        print >> sys.stderr, "Testing SecretServiceKeyring, following password prompts are for this keyring"
+        return keyring.backend.SecretServiceKeyring()
+
+    def test_supported_no_module(self):
+        with ImportKiller('dbus'):
+            with Environ(**self.environ()):
+                self.assertEqual(-1, self.keyring.supported())
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(OSXKeychainTestCase))
     suite.addTest(unittest.makeSuite(GnomeKeyringTestCase))
+    suite.addTest(unittest.makeSuite(SecretServiceKeyringTestCase))
     suite.addTest(unittest.makeSuite(KDEWalletCanceledTestCase))
     suite.addTest(unittest.makeSuite(KDEKWalletTestCase))
     suite.addTest(unittest.makeSuite(KDEKWalletInQApplication))
